// Une mesure analogique dans l'intervalle [0,5]V donne une valeur numérique dans l'intervalle [[0;1023]]

// *** Fonction Tone ****
// tone(pin, fréquence, durée)  -- si le champ durée n'est pas renseigné, la note est infini
// noTone(pin)  -- arrete le son


#include "pitches.h" //bibliothèque de notes
//Définition des pins utilisés pour les différents composants
#define L1_Rouge_Pin 4
#define L1_Vert_Pin 3
#define L1_Bleu_Pin 2
#define HP_Pin 8

//Définition d'un vecteur comprenant l'intégralité des notes fournies par la librairie pitches.h, plus l'indice de la note dans le tableau est grand, plus celle-ci est aigüe
int Notes[] = 
{ NOTE_B0 , NOTE_C1, NOTE_CS1 , NOTE_D1 , NOTE_DS1 , NOTE_E1 , NOTE_F1 , NOTE_FS1 , NOTE_G1 , NOTE_GS1 , NOTE_A1 , NOTE_AS1 , NOTE_B1 , NOTE_C2 , NOTE_CS2 , NOTE_D2 , NOTE_DS2 , NOTE_E2 , NOTE_F2 , NOTE_FS2 , NOTE_G2 , NOTE_GS2 , NOTE_A2 , NOTE_AS2 , NOTE_B2 , NOTE_C3 , NOTE_CS3 , NOTE_D3 , NOTE_DS3 , NOTE_E3 , NOTE_F3 , NOTE_FS3 , NOTE_G3 , NOTE_GS3 , NOTE_A3 , NOTE_AS3 , NOTE_B3 , NOTE_C4 , NOTE_CS4 , NOTE_D4 ,
 NOTE_DS4 , NOTE_E4 , NOTE_F4 , NOTE_FS4 , NOTE_G4 , NOTE_GS4 , NOTE_A4 , NOTE_AS4 , NOTE_B4 , NOTE_C5 , NOTE_CS5 , NOTE_D5 , NOTE_DS5 , NOTE_E5 , NOTE_F5 , NOTE_FS5 , NOTE_G5 , NOTE_GS5 , NOTE_A5 , NOTE_AS5 , NOTE_B5 , NOTE_C6 , NOTE_CS6 , NOTE_D6 , NOTE_DS6 , NOTE_E6 , NOTE_F6 , NOTE_FS6 , NOTE_G6 , NOTE_GS6 , NOTE_A6 , NOTE_AS6 , NOTE_B6 , NOTE_C7 , NOTE_CS7 , NOTE_D7 , NOTE_DS7 , NOTE_E7 , NOTE_F7 , NOTE_FS7 ,
 NOTE_G7 , NOTE_GS7 , NOTE_A7 , NOTE_AS7 , NOTE_B7 , NOTE_C8 , NOTE_CS8 , NOTE_D8 , NOTE_DS8 };

void setup() {
  pinMode(HP_Pin,OUTPUT); //réglage du pin de sortie pour le haut-parleur
  pinMode(L1_Rouge_Pin, OUTPUT); //réglages des pins de sortie pour une LED
  pinMode(L1_Vert_Pin, OUTPUT);
  pinMode(L1_Bleu_Pin, OUTPUT);

  DisplayColor(0,0,0); //permet d'éteindre les LED au démarrage
}

void loop() {
  
  int mesure0 = analogRead(A0); //lecteur d'une valeur sur la pin A0 correspond au signal issu de la plante et traité par la carte
  int mesure1 = analogRead(A1); //lecteur d'une valeur sur la pin A1 correspondant à une tension issue d'un potentiomètre (alimenté par les pins 5V et GND de l'Arduino)

  float note_float = mesure0 * ( (sizeof(Notes)/sizeof(int)-1.0) / 1023.0); //la mesure0 est comprise entre 0 et 1023, et nous cherchons à la faire correspondre à l'une des 89 notes du vecteur Notes[]
  int note = (int) note_float; //la note à jouer est obtenue en arrondissant le résultat précédent, la valeur finale de "note" appartient à [[0;88]] 
  
  float duree_float = mesure1 * ( 9.0 / 1023.0);
  int duree_int = (int) duree_float; //par une méthode similaire à celle réalisée sur mesure0, mesure1 est ramenée à un intervalle de 10 valeurs -> [[0;9]]
  int duree = 5000/(duree_int + 1) //duree correspond à la durée pendant laquelle une note sera jouée, nous avons choisi de travailler sur une fraction de 5000 ms

  //Nous décidons d'allumer les LEDs en nuance de bleu et de rouge; plus le signal de la plante est faible, plus les LEDs seront rouges et plus le signal est fort, plus elles seront bleues
  float couleur_float = mesure0 * ( 255.0 / 1023.0);
  int couleur = (int) couleur_float; // la mesure 0 est ramenée à une valeur comprise dans l'intervalle [[0;255]] afin de coder les couleurs à envoyer aux LEDs
  
  AfficherCouleur( couleur, 0, ~couleur); // ~couleur = 255 - couleur
  
  tone( HP_Pin, Notes[resultat], duree ); //sortie sur la pin HP_Pin, note à jouer, durée de la note en ms
  
  delay(duree*1.30); //delay conseillé pour bien distinguer les notes, proportionnel à la durée de la note
    
  noTone(8); //arrêt de la transmittion de la note
  
}

//La fonction AfficherCouleur permet de prendre le codage classique d'une couleur en héxadécimal, avec chaque couleur primaire codée avec une valeur comprise entre 0 et 255
void AfficherCouleur(byte R, byte G, byte B) 
{
  analogWrite(L1_Rouge_Pin, R); //Nous utilisons des LEDs à cathode commune, aucun besoin d'inverser la valeur R
  analogWrite(L1_Vert_Pinn, G);
  analogWrite(L1_Bleu_Pin, B);
}
